#include <stdio.h>
#include <stdlib.h>
#include "cuda.h"
#include "time.h"

#define SIZE                (1024)
#define THREADS_PER_BLOCK   (16)

__global__ void sum(float *a, float *b, float *c, int w, int h)
{
    int tidx = blockDim.x * blockIdx.x + threadIdx.x;
    int tidy = blockDim.y * blockIdx.y + threadIdx.y;
    if (tidx < w && tidy < h)
        c[w * tidy + tidx] = a[w * tidy + tidx] + b[w * tidy + tidx];
}

int main(void)
{
    float *h_a, *h_b, *h_c;
    float *d_a, *d_b, *d_c;
    
    int w = SIZE;
    int h = SIZE;

	// Rezervacija pomnilnika - gostitelj
	h_a = (float *)malloc(w * h * sizeof(float));
	h_b = (float *)malloc(w * h * sizeof(float));
	h_c = (float *)malloc(w * h * sizeof(float));

	// Inicializacija vektorjev
    srand(time(NULL));
	for (int i = 0; i < w * h; i++)
	{
		h_a[i] = (float)rand()/RAND_MAX;
		h_b[i] = (float)rand()/RAND_MAX;
	}

    // Rezervacija pomnilnika - naprava
    cudaMalloc((void **)&d_a, w * h * sizeof(float));
    cudaMalloc((void **)&d_b, w * h * sizeof(float));
    cudaMalloc((void **)&d_c, w * h * sizeof(float));

    // prenos podatkov na napravo
    cudaMemcpy(d_a, h_a, w * h * sizeof(float), cudaMemcpyHostToDevice);
    cudaMemcpy(d_b, h_b, w * h * sizeof(float), cudaMemcpyHostToDevice);

	// Racunanje
    dim3 threadsperblock(THREADS_PER_BLOCK, THREADS_PER_BLOCK);
    dim3 blockspergrid((w-1)/THREADS_PER_BLOCK + 1, (h-1)/THREADS_PER_BLOCK + 1);

    sum<<<blockspergrid,threadsperblock>>>(d_a, d_b, d_c, w, h);
    cudaError_t err = cudaGetLastError();
    if (err != cudaSuccess) 
    {
        printf("cudaGetDeviceCount error %d\n-> %s\n", err, cudaGetErrorString(err));
        exit(EXIT_FAILURE);
    }


    // prenos podatkov iz naprave
    cudaMemcpy(h_c, d_c, w * h * sizeof(float), cudaMemcpyDeviceToHost);

    // sproscanje pomnilnika
    cudaFree(d_a);
    cudaFree(d_b);
    cudaFree(d_c);

	// Prikaz rezultatov
	int sum = 0;
	for (int i = 0; i < h; i++)
		for (int j = 0; j < w; j++)
			sum += h_c[i*w+j] == h_a[i*w+j] + h_b[i*w+j];
	printf("Delež enakih: %lf\n", sum/(double)(h*w));

	free(h_a);
	free(h_b);
	free(h_c);

	return 0;
}
