#include <stdio.h>
#include <stdlib.h>
#include "cuda.h"

#define SIZE    (1024)

__global__ void sum(float *a, float *b, float *c)
{
    int tid = threadIdx.x;
    c[tid] = a[tid] + b[tid];
}

int main(void)
{
    float *h_a, *h_b, *h_c;
    float *d_a, *d_b, *d_c;
    
	// Rezervacija pomnilnika - gostitelj
	h_a = (float *)malloc(SIZE * sizeof(float));
	h_b = (float *)malloc(SIZE * sizeof(float));
	h_c = (float *)malloc(SIZE * sizeof(float));

	// Inicializacija vektorjev
	for (int i = 0; i < SIZE; i++)
	{
		h_a[i] = i;
		h_b[i] = SIZE-i;
	}

    // Rezervacija pomnilnika - naprava
    cudaMalloc((void **)&d_a, SIZE * sizeof(float));
    cudaMalloc((void **)&d_b, SIZE * sizeof(float));
    cudaMalloc((void **)&d_c, SIZE * sizeof(float));

    // prenos podatkov na napravo
    cudaMemcpy(d_a, h_a, SIZE * sizeof(float), cudaMemcpyHostToDevice);
    cudaMemcpy(d_b, h_b, SIZE * sizeof(float), cudaMemcpyHostToDevice);

	// Racunanje
    sum<<<1,SIZE>>>(d_a, d_b, d_c);
    cudaError_t err = cudaGetLastError();
    if (err != cudaSuccess) 
    {
        printf("cudaGetDeviceCount error %d\n-> %s\n", err, cudaGetErrorString(err));
        exit(EXIT_FAILURE);
    }


    // prenos podatkov iz naprave
    cudaMemcpy(h_c, d_c, SIZE * sizeof(float), cudaMemcpyDeviceToHost);

    // sproscanje pomnilnika
    cudaFree(d_a);
    cudaFree(d_b);
    cudaFree(d_c);

	// Prikaz rezultatov
	for (int i = 0; i < SIZE; i++)
		printf("%lf + %lf = %lf\n", h_a[i], h_b[i], h_c[i]);

	free(h_a);
	free(h_b);
	free(h_c);

	return 0;
}
